from marshmallow import (
    Schema,
    fields,
    validate,
)


class ContextResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    key = fields.String(required=True)
    value = fields.String(validate=not_blank, required=True)



from marshmallow import (
    Schema,
    fields,
)

from ..AssistantCommand.models import AssistantCommandTypeEnum, AssistantCommandStatusEnum


class AssistantCommandResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    user_query_id = fields.Integer(required=True)
    step_number = fields.Integer(required=True)
    type = fields.Enum(AssistantCommandTypeEnum, required=True, by_value=True)
    label = fields.String(required=True)
    status = fields.Enum(AssistantCommandStatusEnum, required=True, by_value=True)
    result = fields.Dict(required=True, allow_none=True)
    created_at = fields.DateTime(dump_only=True)
    updated_at = fields.DateTime(dump_only=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Text,
)
from datetime import datetime
from ...database import Base


class FDALabelModel(Base):
    __tablename__ = 'fda_labels'

    id = Column(Integer, primary_key=True)
    set_id = Column(String(128), nullable=False, index=True)
    doc_id = Column(String(128), nullable=False)
    date = Column(DateTime, nullable=False)
    version = Column(String(20), nullable=False)
    section_id = Column(String(128), nullable=False)
    section_type = Column(String(20), nullable=False)
    section_html_file_id = Column(
        Integer,
        ForeignKey('files.id'),
        nullable=True,
    )
    label_html_file_id = Column(
        Integer,
        ForeignKey('files.id'),
        nullable=False,
    )
    section_text = Column(Text, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
)
from datetime import datetime

from ...database import Base


class DeletedNewswireModel(Base):
    __tablename__ = 'deleted_newswires'

    id = Column(Integer, primary_key=True)
    date = Column(DateTime)
    source = Column(String(128))
    headline = Column(Text)
    s3_key_name = Column(String(255), nullable=False)
    source_link = Column(Text, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)
from datetime import datetime

from sqlalchemy.orm import relationship

from ..AssistantCommand.models import AssistantCommandModel
from ...database import Base


class AssistantUserQueryModel(Base):
    __tablename__ = "assistant_user_queries"

    id = Column(Integer, primary_key=True)
    session_id = Column(
        Integer,
        ForeignKey("assistant_sessions.id"),
        nullable=False,
    )
    value = Column(
        String,
        nullable=False,
    )
    created_at = Column(
        DateTime,
        nullable=False,
        default=lambda: datetime.utcnow(),
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    # These are ORM fields. Don't need to be added in the corresponding migration.
    # https://docs.sqlalchemy.org/en/14/orm/tutorial.html#building-a-relationship
    session = relationship(
        "AssistantSessionModel",
        back_populates="user_queries",
    )
    commands = relationship(
        "AssistantCommandModel",
        order_by=AssistantCommandModel.step_number,
        back_populates="user_query",
    )



from marshmallow import Schema, fields

from ..AssistantUserQuery.schemas import AssistantUserQueryResourceSchema


class AssistantSessionResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    user_id = fields.String(required=True)
    user_queries = fields.Nested(AssistantUserQueryResourceSchema(exclude=("session_id",)), many=True, dump_only=True)
    created_at = fields.DateTime(dump_only=True)
    updated_at = fields.DateTime(dump_only=True)



from sqlalchemy.orm import joinedload

from cs_models.resources.AssistantSession.models import AssistantSessionModel
from cs_models.resources.AssistantSession.schemas import AssistantSessionResourceSchema
from cs_models.resources.AssistantUserQuery.models import AssistantUserQueryModel
from cs_models.resources.AssistantUserQuery.schemas import (
    AssistantUserQueryResourceSchema,
)
from cs_models.resources.AssistantCommand.models import (
    AssistantCommandModel,
    AssistantCommandStatusEnum,
    AssistantCommandTypeEnum,
)
from cs_models.resources.AssistantCommand.schemas import AssistantCommandResourceSchema

from cs_models.database import operations
from sqlalchemy import select, asc

from pprint import pprint


def main():
    with operations.session_scope() as db_session:
        # Get or create Assistant session
        assistant_session, _ = operations.get_or_create(
            session=db_session,
            model=AssistantSessionModel,
            schema=AssistantSessionResourceSchema,
            obj={"user_id": "auth0|1000"},
        )

        # assistant_query, _ = operations.get_or_create(
        #     session=db_session,
        #     model=AssistantUserQueryModel,
        #     schema=AssistantUserQueryResourceSchema,
        #     obj={"session_id": assistant_session.id, "value": "get me 1"},
        # )
        #
        # operations.get_or_create(
        #     session=db_session,
        #     model=AssistantCommandModel,
        #     schema=AssistantCommandResourceSchema,
        #     obj={
        #         "user_query_id": assistant_query.id,
        #         "step_number": 0,
        #         "type": AssistantCommandTypeEnum.SEARCH,
        #         "label": "Search source 1",
        #         "status": AssistantCommandStatusEnum.NOT_STARTED,
        #         "result": None,
        #     },
        # )
        # operations.get_or_create(
        #     session=db_session,
        #     model=AssistantCommandModel,
        #     schema=AssistantCommandResourceSchema,
        #     obj={
        #         "user_query_id": assistant_query.id,
        #         "step_number": 1,
        #         "type": AssistantCommandTypeEnum.EXTRACT,
        #         "label": "Extract something",
        #         "status": AssistantCommandStatusEnum.NOT_STARTED,
        #         "result": None,
        #     },
        # )
        #
        # operations.get_or_create(
        #     session=db_session,
        #     model=AssistantUserQueryModel,
        #     schema=AssistantUserQueryResourceSchema,
        #     obj={"session_id": assistant_session.id, "value": "get me 2"},
        # )

        pprint(AssistantSessionResourceSchema().dump(assistant_session))
        #


if __name__ == "__main__":
    main()



import enum
from datetime import datetime

from sqlalchemy import Column, DateTime, Enum, ForeignKey, Integer, Text
from sqlalchemy.orm import relationship

from ...database import Base


class AssistantCommandTypeEnum(enum.Enum):
    SEARCH = "SEARCH"
    EXTRACT = "EXTRACT"
    SUMMARIZE = "SUMMARIZE"
    QUESTION_AND_ANSWER = "QUESTION_AND_ANSWER"


class AssistantCommandStatusEnum(enum.Enum):
    NOT_STARTED = "NOT_STARTED"
    IN_PROGRESS = "IN_PROGRESS"
    FAILED = "FAILED"
    SUCCESSFUL = "SUCCESSFUL"


class AssistantCommandModel(Base):
    __tablename__ = "assistant_commands"

    id = Column(Integer, primary_key=True)
    user_query_id = Column(
        Integer,
        ForeignKey("assistant_user_queries.id"),
        nullable=False,
    )
    step_number = Column(Integer, nullable=False)
    type = Column("type", Enum(AssistantCommandTypeEnum))
    status = Column("status", Enum(AssistantCommandStatusEnum))
    label = Column(Text, nullable=False)
    result = Column(Text)
    created_at = Column(
        DateTime, nullable=False, default=lambda: datetime.utcnow()
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    # These are ORM fields. Don't need to be added in the corresponding migration.
    # https://docs.sqlalchemy.org/en/14/orm/tutorial.html#building-a-relationship
    user_query = relationship(
        "AssistantUserQueryModel",
        back_populates="commands",
    )



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    String,
    ForeignKey,
    Text,
    Boolean,
)

from ...database import Base


class ProductReviewModel(Base):
    __tablename__ = "product_review"

    id = Column(Integer, primary_key=True)
    intervention_norm_id = Column(
        Integer,
        ForeignKey('intervention_norm.id'),
        nullable=False,
    )
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    concern_type = Column(String(50), nullable=False)
    concern_detail = Column(String(128), nullable=False)
    note = Column(Text, nullable=True)
    references = Column(Text, nullable=True)
    assignee_id = Column(
        Integer,
        ForeignKey('assignees.id'),
        nullable=True,
    )
    reviewer_id = Column(
        Integer,
        ForeignKey('reviewers.id'),
        nullable=True,
    )
    reviewed = Column(
        Boolean,
        nullable=True,
    )
    date_logged = Column(DateTime, nullable=False)
    date_reviewed = Column(DateTime, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy import (
    Column,
    String,
    Integer,
    ForeignKey,
)

from ...database import Base


class PubmedCompanyOutboxModel(Base):
    __tablename__ = 'pubmed_company_outbox'

    id = Column(Integer, primary_key=True)
    pubmed_id = Column(
        Integer,
        ForeignKey('pubmed.id'),
        nullable=False,
    )
    entity_name = Column(String(256), nullable=False)



from marshmallow import Schema, fields

from ..AssistantCommand.schemas import (
    AssistantCommandResourceSchema,
)


class AssistantUserQueryResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    session_id = fields.Integer(required=True)
    value = fields.String(required=True)
    commands = fields.Nested(AssistantCommandResourceSchema(exclude=("user_query_id",)), many=True, dump_only=True)
    created_at = fields.DateTime(dump_only=True)
    updated_at = fields.DateTime(dump_only=True)



from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    Boolean,
    String,
    Text,
)
from datetime import datetime

from ...database import Base


class InsightsQueueModel(Base):
    __tablename__ = 'insights_queue'

    id = Column(Integer, primary_key=True)
    source_type = Column(String(50), nullable=False)
    source_id = Column(
        Integer,
        nullable=False,
    )
    processed = Column(
        Boolean,
        nullable=True,
    )
    error = Column(Text, nullable=True)
    submitted = Column(Boolean, nullable=True)
    submitted_date = Column(DateTime, nullable=True)
    historical = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class FDALabelResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    set_id = fields.String(required=True)
    doc_id = fields.String(required=True)
    date = fields.DateTime(required=True)
    version = fields.String(required=True)
    section_id = fields.String(required=True)
    section_text = fields.String(allow_none=True)
    section_type = fields.String(required=True)
    section_html_file_id = fields.Integer(allow_none=True)
    label_html_file_id = fields.Integer(required=True)
    updated_at = fields.DateTime(dump_only=True)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    ForeignKey,
)

from ...database import Base


class OrangeBookProductSubsidiaryModel(Base):
    __tablename__ = "orange_book_products_subsidiaries"

    id = Column(Integer, primary_key=True)
    orange_book_product_id = Column(
        Integer,
        ForeignKey('orange_book_products.id'),
        nullable=False,
    )
    subsidiary_id = Column(
        Integer,
        ForeignKey('subsidiaries.id'),
        nullable=False,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class ActionLogResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    user_id = fields.String(required=True)
    action_type = fields.String(required=True)
    endpoint = fields.String(required=True)
    payload = fields.String(required=True)
    table_name = fields.String(allow_none=True)
    table_id = fields.Integer(allow_none=True)
    transformation = fields.String(allow_none=True)
    timestamp = fields.DateTime(dump_only=True)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    ForeignKey,
    String,
    Boolean,
)

from ...database import Base


class InterventionDataModel(Base):
    __tablename__ = "intervention_data"

    id = Column(Integer, primary_key=True)
    source_type = Column(
        String(50),
        nullable=False
    )
    date = Column(DateTime, nullable=True)
    news_id = Column(
        Integer,
        ForeignKey('newswires.id'),
        nullable=True,
    )
    pubmed_id = Column(
        Integer,
        ForeignKey('pubmed.id'),
        nullable=True,
    )
    file_id = Column(
        Integer,
        ForeignKey('files.id'),
        nullable=True,
    )
    title = Column(String(255), nullable=True)
    orig_file_url = Column(String(255), nullable=True)
    is_deleted = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



"""Models for storing Mindgram assistant sessions."""
from datetime import datetime

from sqlalchemy import Column, DateTime, Integer, String
from sqlalchemy.orm import relationship

from ..AssistantUserQuery.models import AssistantUserQueryModel
from ...database import Base


class AssistantSessionModel(Base):
    """Model for storing Assistant Session."""

    __tablename__ = "assistant_sessions"

    id = Column(Integer, primary_key=True)
    user_id = Column(String(128), unique=True, nullable=False)
    created_at = Column(
        DateTime,
        nullable=False,
        default=lambda: datetime.utcnow(),
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    # These are ORM fields. Don't need to be added in the corresponding migration.
    # https://docs.sqlalchemy.org/en/14/orm/tutorial.html#building-a-relationship
    user_queries = relationship(
        "AssistantUserQueryModel",
        order_by=AssistantUserQueryModel.created_at,
        back_populates="session",
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class DeletedNewswireResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    date = fields.DateTime(required=True)
    source = fields.String(required=True)
    headline = fields.String(required=True)
    s3_key_name = fields.String(required=True)
    source_link = fields.String(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
)
from datetime import datetime

from ...database import Base


class ReviewerModel(Base):
    __tablename__ = 'reviewers'

    id = Column(Integer, primary_key=True)
    user_id = Column(String(128), nullable=False)
    user_name = Column(String(128), nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



# pylint: disable=unnecessary-lambda
from sqlalchemy import (
    Column,
    String,
    Text,
)

from ...database import Base


class ContextModel(Base):
    __tablename__ = "contexts"

    key = Column(String(64), primary_key=True)
    value = Column(Text, nullable=False)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class MeetingResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    meeting_name = fields.String(required=True)
    start_date = fields.DateTime(required=True)
    end_date = fields.DateTime(required=True)
    from_website = fields.Boolean(allow_none=True)
    meeting_bucket_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


INSTALL_REQUIRES = [
    "marshmallow==3.19.0",
    "PyMySQL==1.0.2",
    "python-dateutil==2.8.2",
    "pytz==2022.1",
    "SQLAlchemy==1.4.35",
    "boto3==1.21.46",
    "botocore==1.24.46",
    "s3transfer==0.5.2",
    "six==1.16.0",
    "urllib3>=1.26.9",
    "dnspython==2.2.1",
    "pymongo==4.1.1",
    "psycopg2-binary==2.9.3",
    "sshtunnel==0.4.0",
]

if __name__ == "__main__":
    setuptools.setup(
        name="cs-models",
        version="0.0.550.dev6",
        author="Shrey Verma",
        author_email="shrey@mindgram.ai",
        description="MySQL db models",
        # long_description=long_description,
        # long_description_content_type='text/markdown',
        url="https://github.com/mindgram/cs-models",
        packages=setuptools.find_packages(where="src"),
        package_dir={"": "src"},
        classifiers=[
            "Programming Language :: Python :: 3",
        ],
        install_requires=INSTALL_REQUIRES,
        python_requires="~=3.9",
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Boolean,
    Float,
)
from datetime import datetime

from ...database import Base


class CompanyFilingTypeReviewModel(Base):
    __tablename__ = 'company_filing_type_review'

    id = Column(Integer, primary_key=True)
    company_file_id = Column(
        Integer,
        ForeignKey('company_filings.id'),
        nullable=False,
    )
    filing_type = Column(String(255), nullable=True)
    filing_type_score = Column(Float, nullable=True)
    reviewed = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )
